package com.thoughtworks.rettig.dataLoader;

import java.lang.reflect.*;
import java.util.*;
import junit.framework.*;
import java.util.*;
import com.thoughtworks.rettig.util.*;

/**
* This class uses a static method to construct an object using a data iterator.
* Relies heavily on reflection and recursion to dynamically create the objects.
**/

public class ReflectiveObjectLoader {

	private final Loggable log;
	private final LoadManager manager;

	private ReflectiveObjectLoader(LoadManager loadManager, Loggable myLog){
	    manager = loadManager;
	    this.log = myLog;
	}

	/** 
	* Uses the data iterator to create an array of arbitrary length.
	**/
	final private Object initializeArray(Class objClass)throws
	    java.lang.IllegalAccessException,
	    java.lang.reflect.InvocationTargetException,
	    java.lang.InstantiationException{

		log.logMsg(5, "Array Component: " + objClass.getComponentType().getName());

		Class subObj = objClass.getComponentType();

		if (subObj.getName().equals(int.class.getName())) {
			return constructIntArray(subObj);
		} else {
			return constructObjectArray(subObj);
		}

		//add support for other types of arrays here
	}

	final private Object constructObjectArray(Class arrayType)throws
	    java.lang.IllegalAccessException,
	    java.lang.reflect.InvocationTargetException,
	    java.lang.InstantiationException{

		Vector v = new Vector();

		while (manager.hasMoreDataForArray()) {
			// recursive construction
			v.add(initializeInstance(arrayType));
		}

		Object[] newArray = (Object[]) Array.newInstance(arrayType, v.size());
		v.copyInto(newArray);
		return newArray;

	}

	final private Object constructIntArray(Class arrayType){
		Vector v = new Vector();

		LoadableClass toLoad = manager.getLoadableForClass(arrayType);
		
		while (manager.hasMoreDataForArray()) {
			v.add(toLoad.load());
		}

		Object newArray = Array.newInstance(arrayType, v.size());

		for (int i =0; i< Array.getLength(newArray); i++) {
			Array.setInt(newArray,i, ((Integer)v.get(i)).intValue());
		}

		return newArray;

	}

	final public static Object initializeInstance(Class obj, LoadManager manager)throws
	    java.lang.IllegalAccessException,
	    java.lang.reflect.InvocationTargetException,
	    java.lang.InstantiationException{

	    Loggable emptyLog = new Loggable(){
		    public void logMsg(int level, String msg) {
			// do nothing
		    }
		    };
	    return initializeInstance(obj, manager, emptyLog);

	}

	final public static Object initializeInstance(Class obj, LoadManager manager, Loggable log)throws
	    java.lang.IllegalAccessException,
	    java.lang.reflect.InvocationTargetException,
	    java.lang.InstantiationException{

	    ReflectiveObjectLoader loader = new ReflectiveObjectLoader(manager, log);

	    return loader.initializeInstance(obj);
	}

	/**
	* Creates an instance of the Class object passed into the method, using
	* the data iterator. Basically the function knows how to initialize four
	* Objects: Arrays, Strings,ints, and Objects.
	*/
	final private Object initializeInstance(Class obj)throws
	    java.lang.IllegalAccessException,
	    java.lang.reflect.InvocationTargetException,
	    java.lang.InstantiationException{
  

		log.logMsg(Loggable.DEBUG, "Initializing: " + obj.getName());

		LoadableClass loadClass = manager.getLoadableForClass(obj);

		if(loadClass.isArray()) {
		    //System.out.println("is array");
			return initializeArray(obj);
		}

		if(loadClass.canLoad()){
		    //System.out.println("Can load");
		    log.logMsg(Loggable.DEBUG, "Is Loadable");
		    return loadClass.load();
		}
     		
		if(loadClass.canRecurse()){
		    //System.out.println("Can recurse");
		    return constructObject(loadClass);//recursive construction
		}

		Assert.fail("Do not know how to handle Class: " + obj.getName());
		return null;

	}
	/**
	* Recursively constructs an object.
	**/
	final private Object constructObject(LoadableClass obj)throws
	    java.lang.IllegalAccessException,
	    java.lang.reflect.InvocationTargetException,
	    java.lang.InstantiationException{

		log.logMsg(Loggable.DEBUG, "Attempting to construct: " + obj.getName());

		Constructor construct = obj.getRecursiveConstructor();

		Class[] parameters = construct.getParameterTypes();

		Object[] constructedParams = new Object[parameters.length];
  
		for (int i = 0; i< parameters.length; i++) {

			log.logMsg(Loggable.DEBUG, "Param " + i + " = " + parameters[i].getClass().getName());
			//System.out.println(parameters[i].getName());
			//if(parameters[i].isPrimitive()){
			    //System.out.println("is primitive");
			//}
			constructedParams[i] = initializeInstance(parameters[i]);              
		}

		return construct.newInstance(constructedParams);
	}

	final private Constructor getFullConstructor(Class obj){
		Constructor[] constructors = obj.getConstructors();
		  // Must know "somehow" which constructor to call.
		  // Here we'll assume the first constructor is always the full constructor.
		return constructors[0];
	}

}


