package com.thoughtworks.rettig.example;

import com.thoughtworks.rettig.util.*;
import com.thoughtworks.rettig.example.generated.*;
import com.thoughtworks.rettig.dataLoader.*;

public class TestPerformance {
	private final static String file = "testFiles\\purchaseOrder\\PurchaseOrder.txt";

	public static void main(String[] args)throws Exception{
		int number = 100000;

		if (args.length == 1) {
			number = Integer.parseInt(args[0]);
		}
		System.out.println("Number of Iterations: " + number);
		SimpleFileIterator iter = new SimpleFileIterator(file);
		testReflection(number, iter);
		testGenerated(number, iter);  	
	}

	private static void testReflection(int number, SimpleFileIterator iter)throws Exception{
		long start = System.currentTimeMillis();
		FindMax max = new FindMax();
		max.start();

		LoadManager manager = new SimpleFileLoadManager(iter);
		for (int i = 0; i<number; i++) {   
			PurchaseOrder obj = (PurchaseOrder)ReflectiveObjectLoader.initializeInstance(PurchaseOrder.class, manager);
			iter.reset();
		}
		max.endRun();
		max.join();
		long end = System.currentTimeMillis();
		printResults("Reflection",start,end, max.getMaxUsage());  
	}

	private static void testGenerated(int number, SimpleFileIterator iter)throws Exception{
		long start = System.currentTimeMillis();
		FindMax max = new FindMax();
		max.start();

		for (int i = 0; i<number; i++) {
			PurchaseOrder po = PurchaseOrderLoader.loadPurchaseOrder(iter);
			iter.reset();
		}
		max.endRun();
		max.join();
		long end = System.currentTimeMillis();
		printResults("Generated",start,end, max.getMaxUsage());  
	}


	private static void printResults(String msg, long start, long end, long maxUsage){
		System.out.println("\n"+ msg);
		System.out.println("Total time: " + (end - start));
		System.out.println("Max Memory Used: " + maxUsage);
	}
}

class FindMax extends Thread {
	private long maxUsage = 0;
	boolean isRunning = true;
	public void run() {
		try {
			while (isRunning) {
				checkMemUsage();
				Thread.currentThread().sleep(15);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void endRun(){
		isRunning = false;
	}


	public long getMaxUsage(){
		return maxUsage;
	}

	private void checkMemUsage(){
		Runtime rt = Runtime.getRuntime();
		long used = rt.totalMemory() - rt.freeMemory();
		if (used > maxUsage) {
			maxUsage = used;
		}
	}
}

