package com.thoughtworks.rettig.loadGenerator;

import java.util.*;
import junit.framework.*;

/**
* Used to construct a Java source file and return it as a string.
**/
public class CodeBuilder {

	Hashtable allMethods = new Hashtable();

	private String packageName ="";
	private Class toConvert;
	private MethodFactory factory;

	private CodeBuilder(String pack, Class toConvert, MethodFactory fact){
	    this.toConvert = toConvert;
	    this.factory = fact;
	    allMethods.put(toConvert, factory.get(toConvert, this));
	    packageName = pack;
	}

	public String getPackage(){
	    if(packageName.equals("")){
		return "";
	    }
	    else{
		return "package " + packageName + ";\n\n";
	    }
	}
	/**
	* Returns a list of all the required imports for the code
	**/
	
	private String[] getImports(){
	    return new String[0];
	}

	/**
	* returns the imports as string
	**/
	private String printImports(){
	    String[] imports = getImports();
	    String info = "";
	    for (int i = 0; i < imports.length; i++) {
		 info += "import " + imports[i] + ";\n";
	    }
	    return info +"\n";
	}


	/**
	* Returns a MethodCode object for a specific class
	* if the method does not exist, then it is constructed
	**/
	public MethodCode getMethod(Class classObj){
	    //System.out.println("Finding Method: " + classObj.getName());
	
		MethodCode m = (MethodCode)allMethods.get(classObj);
		if (m != null) {
			return m;
		} else {
			allMethods.put(classObj, factory.get(classObj, this));
			return getMethod(classObj);
		}

	}

	/** 
	* returns the load class name
	**/
	public String getLoadClassName(){
		return getLoadClassName(toConvert);
	}

	/**
	* Used to generate a file name for a particular class
	**/
	public static String convertToFileName(String fullClass){
	    int num = fullClass.lastIndexOf(".");
	    return fullClass.substring(num+1);
	}

	/**
	* generates a load class name for a particular class
	**/
	public static String getLoadClassName(Class theClass){
	    String name = "";
	    if(theClass.isArray()){
		name = theClass.getComponentType().getName() +"Array";
	    }
	    else{
		name = theClass.getName();
	    }
	      return convertToFileName(name) + "Loader";
	}
    
	/**
	* Uses the parameters to generate a Java source file as a string
	**/
	public static String getCode(Class toConvert, String packageName, MethodFactory factory){
		CodeBuilder code = new CodeBuilder(packageName, toConvert, factory);
    		String  body = code.getPackage();
		body += code.printImports();
		body += "public class " + code.getLoadClassName() + "{\n";
		body += code.printMethods() + "\n}\n";
		return body;
	}

   	/**
	* Returns all the methods of the source as a string
	**/
	public String printMethods(){
		Enumeration values = allMethods.elements();

		String methods = "";
		int num = 0;
		while (values.hasMoreElements()) {
			MethodCode m = (MethodCode)values.nextElement();
			//System.out.println("Adding method" + (num++) + m.getMethodName()); 
			methods += m.getBody() + "\n";
		}

		return methods;

	}


}

