package com.thoughtworks.rettig.loadGenerator;

import java.lang.reflect.*;
import java.io.*;
import java.util.*;

/**
* Utitlity to generate a source file and write it as a file
**/

public class LoadWriter {

    private final static String header = "/** Generated Load File. Do not edit!!! **/\n\n";

    public static void main(String[] args)throws Exception{
	String[] classes = new String[args.length-1];
	System.arraycopy(args,1, classes, 0,args.length-1);
	write(args[0], classes, new MethodFactory());
    }

    public static void write(String packageName, String[] classArgs, MethodFactory factory)throws Exception{

	   for(int i = 0; i< classArgs.length; i++){
 		String className = convertArg(classArgs[i]);
		Class toLoad = Class.forName(className);
		String theCode = CodeBuilder.getCode(toLoad, packageName, factory);
		theCode = header + theCode;
		String fileName = CodeBuilder.getLoadClassName(toLoad);
		System.out.println("Writing " + fileName);    	
		writeFile(theCode, fileName+".java", packageName);
	   }
    }

    private static String convertArg(String arg){
	if(arg.endsWith("[]")){
	    String name = "[L" + arg.substring(0, arg.indexOf("[")) +";";
	    return name;
	}
	return arg;
    }

    private static void writeFile(String theCode, String fileName, String packageName)throws Exception{
	String path = convertPackageToPath(packageName);
	createDirectories(path);
	File f = new File(path + fileName);
	FileOutputStream out = new FileOutputStream(f);
	out.write(theCode.getBytes());

    }

    private static String convertPackageToPath(String packageName){
	String path = packageName.replace('.', File.separatorChar) + File.separator;
	return path;
    }

    private static void createDirectories(String path){
		File f = new File(path);
		if(!f.exists()){
       		 f.mkdirs();
		}
      
    }


}
