package com.thoughtworks.rettig.loadGenerator;

import java.lang.reflect.*;
import junit.framework.*;

/**
* Represents a Method of a class
* Used to construct Source code.
**/
public abstract class MethodCode {
	private final static String access = "final public static"; 
	protected Class returnType;
	private final String  impl;

	public MethodCode(Class type, CodeBuilder builder){
		returnType = type;
		impl = getImpl(builder);
	}

	protected static String convertToFileName(String fullClass){
	 int num = fullClass.lastIndexOf(".");
	 return fullClass.substring(num+1);
        }

	protected String getMethodComment(){
	   return "/** Generated Load method for " + getReturnString() + "**/\n";
	}

	/**
	* These are the arguments that the method requires to run
	**/
	public MethodParameter[] getInputParameters(){
	    return new MethodParameter[0];
	}

	/**
	* These parameters are created during the execution of the method.
	* These might be needed by any method that is called within the
	* body of the method.
	**/
	public MethodParameter[] getInstanceParameters(){
	    return new MethodParameter[0];
	}

	/**
	* Returns the methodName
	**/
	public String getMethodName(){
		return "load" + convertToFileName(returnType.getName());
	}

	public String getBody(){
	    return getMethodComment() + impl;
	}

	/**
	* Generates a call to this method, so that it can be used
	* in the body of other functions
	**/
	public String getCall(MethodParameter[] parentParams){
		return getMethodName() + "(" + createParameterList(parentParams) +")";       
	}

	protected String getCall(CodeBuilder builder, Class theClass){
	    MethodCode m = builder.getMethod(theClass);
	    return m.getCall(getInstanceParameters());
	}

	private String createParameterList(MethodParameter[] parentParameters){
	    MethodParameter[] parameters = getInputParameters();
	    String list = "";
	    for(int i =0; i <parameters.length; i++){
		if(i!=0){
		    list += ", ";
		}

		list += findName(parameters[i].getType(), parentParameters);
	    }

	    return list;
	}

	private static String findName(Class type, MethodParameter[] parent){
	    for(int i = 0; i <parent.length; i++){
		if(parent[i].getType() == type){
		    return parent[i].getName();
		}
	    }

	    Assert.fail("Must have parameter to match: " + type.getName());
	    return null;
	}


	public String getReturnString(){
	    return returnType.getName();
	}

	protected String getImpl(CodeBuilder builder){
	    String code = access + " " + getReturnString()  + " " + getCallSignature() + "{\n";
	    code += getImplBody(builder);
	    code += "}\n";
	    return code;
	}

	private String getCallSignature(){
	    String call = getMethodName() + "(";
      
	    MethodParameter[] input = getInputParameters();
	    for(int i = 0; i < input.length; i++){
		if(i !=0){
		   call += ", ";
		}
       		call += input[i].getTypeName() + " " + input[i].getName();
	    }
	     
	    call +=")";
	    return call;

	}

	/**
	* Must be implemented in subclasses to define the body of the method
	**/
	abstract protected String getImplBody(CodeBuilder builder);
}


