package com.thoughtworks.rettig.loadGenerator;

import java.lang.reflect.*;
import com.thoughtworks.rettig.util.SimpleFileIterator;

/**
* Uses reflection to dynamically create source code for an IDL defined object.
* The generated method parses data from a file into a newly created object
**/

public class MethodForObject extends MethodCode {
	private final static MethodParameter param = new MethodParameter(SimpleFileIterator.class, "parser");

	public MethodForObject(Class type, CodeBuilder builder){
		super(type, builder);
	}

	public MethodParameter[] getInputParameters(){
		return new MethodParameter[]{
			param
		};
	}

	public MethodParameter[] getInstanceParameters(){
		return getInputParameters();
	}


	private Class[] getParameters(){
    		//System.out.println(returnType.getName());
		Constructor[] allConstruct = returnType.getConstructors();
		Constructor construct = allConstruct[0];
		return construct.getParameterTypes();
	}

	/**
	* Generates the actual code that will become part of the source file.
	* Reflection is used to determine the constructor and the fields that are 
	* required to parse the data into the object
	**/
	protected String getImplBody(CodeBuilder builder){

		Class[] parameters = getParameters();

		Object[] constructedParams = new Object[parameters.length];
		String code = "return new " + returnType.getName() + "(";

		for (int i = 0; i< parameters.length; i++) {
			if (i>0) {
				code +=", ";
			}
			
			code += builder.getMethod(parameters[i]).getCall(getInstanceParameters()); 
	
		}

		code +=");" + "\n";
		return code;
	}

}
